import re
from vegas import importvegas
from errortools import *
import os


def getplots(s):
    if type(s) == dict:
        return list(set(s.keys()) - {
            'SHA','chi2a','iteration','msg','value','time'
        })
    elif type(s) == list:
        p = set(tuple(getplots(i)) for i in s)
        if len(p) != 1:
            raise KeyError("Plots aren't consistent")
        return list(p.pop())


def importreg(r, folder='.', filenames=None,
              merge={}, types=[int,float,float]):
    if filenames is None:
        filenames = os.listdir(folder)
    dic = {}
    for filename in filenames:
        m = re.match(r, filename)
        if m:
            v = importvegas(os.path.join(folder, filename))
            for p, n in merge.iteritems():
                if n > 0:
                    v[p] = mergebins(v[p], n)

            if len(types) == len(m.groups()):
                k = tuple(f(v) for v,f in zip(m.groups(), types))
            else:
                k = m.groups()
            dic[k] = v

    return dic


def pattern(piece='.*', flavour='.*', obs='', folderp='.*'):
    if len(obs) > 0:
        return (
            folderp + piece + '_' + flavour + '_S(\d+)X([\d\.]+)D([\d\.]+).*'
        )
    else:
        return (
            folderp + piece + '_' + flavour + '_S(\d+)X([\d\.]+)D([\d\.]+).*'
        )


def mergeset(s):
    if len(s) == 0:
        return {}
    if len(s) == 1:
        return s[0]

    c, v = mergenumbers([i['value'] for i in s],True)
    dic = {
        'time': sum([i['time'] for i in s]),
        'value': v,
        'chi2a': [c, [i['chi2a'] for i in s]]
    }

    for plot in getplots(s):
        dic[plot] = mergeplots([i[plot] for i in s])

    return dic


def mergeseeds(s, key=lambda x: (x[1], x[2])):
    groups = set(map(key, s.keys()))

    dic = {}
    for group in groups:
        ss = [
            s[k]
            for k in s.keys()
            if key(k) == group
        ]
        dic[group] = mergeset(ss)

    return dic


def addsets(s):
    if len(s) == 0:
        return {}

    dic = {
        'time': sum([i['time'] for i in s]),
        'value': plusnumbers([i['value'] for i in s]),
        'chi2a': [i['chi2a'] for i in s]
    }

    for plot in getplots(s):
        dic[plot] = combineNplots(addplots, [i[plot] for i in s])

    return dic


def multiintersect(lists):
    if len(lists) == 0:
        return []
    inter = set(lists[0])
    for i in lists[1:]:
        inter = inter.intersection(set(i))
    return list(inter)


def mergefks(*sets):
    paras = multiintersect([i.keys() for i in sets])
    ans = []
    for para in paras:
        ans.append(addsets([i[para] for i in sets]))
    return mergeset(ans)
